@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -421,22 +421,33 @@ public class AdminParser extends CLIParser {
     }
 
     private void parseDynamicOption(String arg) {
-        // TODO Auto-generated method stub
+        
+        String namespace = null;
+        String name = null;
+        String value = null;
+       
         int slashPos = arg.indexOf('/');
-        int equalPos = arg.indexOf('=');
         
-        if(slashPos == -1) {
-            return;
+        
+        if(arg.startsWith("--remove-")) {
+            namespace = arg.substring(9, slashPos);
+            name = arg.substring(slashPos+1);
+        } else {
+            int equalPos = arg.indexOf('=');
+            
+            if(slashPos == -1) {
+                return;
+            }
+            
+            if(equalPos == -1) {
+                return;
+            }
+            
+            namespace = arg.substring(2,slashPos);
+            name = arg.substring(slashPos+1, equalPos);
+            value = arg.substring(equalPos+1);
         }
         
-        if(equalPos == -1) {
-            return;
-        }
-        
-        String namespace = arg.substring(2,slashPos);
-        String name = arg.substring(slashPos+1, equalPos);
-        String value = arg.substring(equalPos+1);
-        
         getDynamicMap(namespace).put(name, value);
     }
 
@@ -456,7 +467,7 @@ public class AdminParser extends CLIParser {
         if(slashPos == -1) {
             return false;
         }
-        return slashPos < arg.indexOf('=');
+        return slashPos < arg.indexOf('=') || arg.startsWith("--remove-");
     }
 
 
