@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -55,10 +55,11 @@ import java.util.ArrayList;
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.Map;
 import com.openexchange.admin.console.AdminParser;
+import com.openexchange.admin.console.AdminParser.NeededQuadState;
 import com.openexchange.admin.console.CLIOption;
 import com.openexchange.admin.console.ServiceLoader;
-import com.openexchange.admin.console.AdminParser.NeededQuadState;
 import com.openexchange.admin.console.context.extensioninterfaces.ContextConsoleCommonInterface;
 import com.openexchange.admin.console.context.extensioninterfaces.ContextConsoleCreateInterface;
 import com.openexchange.admin.console.exception.OXConsolePluginException;
@@ -107,7 +108,7 @@ public abstract class ContextAbstraction extends UserAbstraction {
             return required;
         }
 
-        public void setRequired(boolean required) {
+        public void setRequired(final boolean required) {
             this.required = required;
         }
 
@@ -196,7 +197,7 @@ public abstract class ContextAbstraction extends UserAbstraction {
         }
     }
     
-    protected void extensionConstantProcessing(HashMap<String, CSVConstants> constantsMap) {
+    protected void extensionConstantProcessing(final HashMap<String, CSVConstants> constantsMap) {
         // We don't check for subclasses being null here because if someone has forgotten
         // to set the options he will directly fix it and thus there no need for the
         // future to check everytime
@@ -255,7 +256,7 @@ public abstract class ContextAbstraction extends UserAbstraction {
                 public ArrayList<String> getData(final Context ctx) {
                     return getHumanReableDataOfAllExtensions(ctx, parser);
                 }
-            }));
+            }, false));
         }
     
         final ArrayList<String> humanReadableColumnsOfAllExtensions = getHumanReadableColumnsOfAllExtensions(parser);
@@ -296,6 +297,7 @@ public abstract class ContextAbstraction extends UserAbstraction {
         columns.add("used_quota");
         columns.add("name");
         columns.add("lmappings");
+        columns.add("attributes");
         columns.addAll(getCSVColumnsOfAllExtensions(parser));
         final ArrayList<ArrayList<String>> data = new ArrayList<ArrayList<String>>();
     
@@ -305,7 +307,7 @@ public abstract class ContextAbstraction extends UserAbstraction {
                     return getCSVDataOfAllExtensions(ctx_tmp, parser);
                 }
 
-            }));
+            }, true));
         }
     
         doCSVOutput(columns, data);
@@ -399,20 +401,21 @@ public abstract class ContextAbstraction extends UserAbstraction {
         return fs;
     }
 
+    @Override
     protected Context getContext(final String[] nextLine, final int[] idarray) throws InvalidDataException, ParseException {
         final Context context = super.getContext(nextLine, idarray);
         setValue(nextLine, idarray, ContextConstants.contextname, new MethodStringClosure() {
-            public void callMethod(String value) throws ParseException, InvalidDataException {
+            public void callMethod(final String value) throws ParseException, InvalidDataException {
                 context.setName(value);
             }
         });
         setValue(nextLine, idarray, ContextConstants.lmapping, new MethodStringClosure() {
-            public void callMethod(String value) throws ParseException, InvalidDataException {
+            public void callMethod(final String value) throws ParseException, InvalidDataException {
                 context.addLoginMapping(value);
             }
         });
         setValue(nextLine, idarray, ContextConstants.quota, new MethodStringClosure() {
-            public void callMethod(String value) throws ParseException, InvalidDataException {
+            public void callMethod(final String value) throws ParseException, InvalidDataException {
                 try {
                     context.setMaxQuota(Long.valueOf(value));
                 } catch (final NumberFormatException e) {
@@ -447,7 +450,7 @@ public abstract class ContextAbstraction extends UserAbstraction {
         return new ArrayList<String>();
     }
 
-    private ArrayList<String> makeData(final Context ctx, final ClosureInterface iface) {
+    private ArrayList<String> makeData(final Context ctx, final ClosureInterface iface, final boolean csv) {
         final ArrayList<String> srv_data = new ArrayList<String>();
         srv_data.add(String.valueOf(ctx.getId()));
     
@@ -501,11 +504,44 @@ public abstract class ContextAbstraction extends UserAbstraction {
         } else {
             srv_data.add(null);
         }
+
+        if (csv) {
+            final StringBuilder attrs = new StringBuilder();
+            final Map<String, Map<String, String>> attributes = ctx.getUserAttributes();
+            for (final Map.Entry<String, Map<String, String>> entry : attributes.entrySet()) {
+                final String namespace = entry.getKey();
+                for (final Map.Entry<String, String> attribute : entry.getValue().entrySet()) {
+                    attrs.append(namespace);
+                    attrs.append('/');
+                    attrs.append(attribute.getKey());
+                    attrs.append('=');
+                    attrs.append(attribute.getValue());
+                    attrs.append(',');
+                }
+            }
+            if (attrs.length() != 0) {
+                attrs.setLength(attrs.length() - 1);
+            }
+            srv_data.add(attrs.toString());
+        }
     
         srv_data.addAll(iface.getData(ctx));
     
         return srv_data;
     }
+    
+    protected void applyDynamicOptionsToContext(final AdminParser parser, final Context ctx) {
+        final Map<String, Map<String, String>> dynamicArguments = parser.getDynamicArguments();
+        for(final Map.Entry<String, Map<String, String>> namespaced : dynamicArguments.entrySet()) {
+            final String namespace = namespaced.getKey();
+            for(final Map.Entry<String, String> pair : namespaced.getValue().entrySet()) {
+                final String name = pair.getKey();
+                final String value = pair.getValue();
+                
+                ctx.setUserAttribute(namespace, name, value);
+            }
+        }
+    }
 
 }
 
