@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -85,6 +85,8 @@ public abstract class CreateCore extends ContextAbstraction {
         setContextQuotaOption(parser, true);
         
         setFurtherOptions(parser);
+        
+        parser.allowDynamicOptions();
     }
     
     protected final void commonfunctions(final AdminParser parser, final String[] args) {
@@ -123,6 +125,9 @@ public abstract class CreateCore extends ContextAbstraction {
                 parseAndSetContextQuota(parser, ctx);
                 
                 parseAndSetExtensions(parser, ctx, auth);
+                
+                // Dynamic Options
+                applyDynamicOptionsToContext(parser, ctx);
             } catch (final RuntimeException e) {
                 printError(null, null, e.getClass().getSimpleName() + ": " + e.getMessage(), parser);
                 sysexit(1);
@@ -153,7 +158,7 @@ public abstract class CreateCore extends ContextAbstraction {
         oxlgn.login(auth);
         
         final CSVReader reader = new CSVReader(new FileReader(filename), ',', '"');
-        int[] idarray = csvParsingCommon(filename, reader);
+        int[] idarray = csvParsingCommon(reader);
         int linenumber = 2;
         String [] nextLine;
         lookupRMI();
