@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,6 +49,8 @@
 
 package com.openexchange.admin.osgi;
 
+import java.rmi.Remote;
+import java.rmi.registry.Registry;
 import java.util.Dictionary;
 import java.util.Stack;
 
@@ -62,9 +64,11 @@ import org.osgi.framework.ServiceListener;
 import org.osgi.util.tracker.ServiceTracker;
 
 import com.openexchange.admin.daemons.AdminDaemon;
+import com.openexchange.admin.daemons.osgi.RMITracker;
 import com.openexchange.admin.exceptions.OXGenericException;
 import com.openexchange.admin.plugins.OXUserPluginInterface;
 import com.openexchange.admin.services.AdminServiceRegistry;
+import com.openexchange.config.cascade.ConfigViewFactory;
 import com.openexchange.context.ContextService;
 import com.openexchange.database.CreateTableService;
 import com.openexchange.mailaccount.MailAccountStorageService;
@@ -80,6 +84,7 @@ public class Activator implements BundleActivator {
     private Stack<ServiceTracker> trackers = new Stack<ServiceTracker>();
 
     public void start(BundleContext context) throws Exception {
+        
         trackers.push(new ServiceTracker(
             context,
             PipesAndFiltersService.class.getName(),
@@ -98,6 +103,15 @@ public class Activator implements BundleActivator {
                 context,
                 AdminServiceRegistry.getInstance(),
                 MailAccountStorageService.class)));
+        
+        trackers.push(new ServiceTracker(
+            context,
+            ConfigViewFactory.class.getName(),
+            new RegistryServiceTrackerCustomizer<ConfigViewFactory>(
+                context,
+                AdminServiceRegistry.getInstance(),
+                ConfigViewFactory.class)));
+        
         trackers.push(new ServiceTracker(context, CreateTableService.class.getName(), new CreateTableCustomizer(context)));
         for (int i = trackers.size() - 1; i >= 0; i--) {
             trackers.get(i).open();
@@ -118,6 +132,12 @@ public class Activator implements BundleActivator {
             throw e;
         }
         this.daemon.initRMI(context);
+        
+        Registry registry = AdminDaemon.getRegistry();
+        RMITracker rmiTracker = new RMITracker(context, registry);
+        trackers.push(rmiTracker);
+        rmiTracker.open();
+        
 
         if (log.isInfoEnabled()) {
             final Dictionary<?, ?> headers = context.getBundle().getHeaders();
