@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -48,6 +48,8 @@
  */
 package com.openexchange.admin.rmi;
 
+import java.rmi.Remote;
+import java.rmi.RemoteException;
 import com.openexchange.admin.rmi.dataobjects.Context;
 import com.openexchange.admin.rmi.dataobjects.Credentials;
 import com.openexchange.admin.rmi.dataobjects.User;
@@ -55,12 +57,9 @@ import com.openexchange.admin.rmi.dataobjects.UserModuleAccess;
 import com.openexchange.admin.rmi.exceptions.DatabaseUpdateException;
 import com.openexchange.admin.rmi.exceptions.InvalidCredentialsException;
 import com.openexchange.admin.rmi.exceptions.InvalidDataException;
+import com.openexchange.admin.rmi.exceptions.NoSuchContextException;
 import com.openexchange.admin.rmi.exceptions.NoSuchUserException;
 import com.openexchange.admin.rmi.exceptions.StorageException;
-import com.openexchange.admin.rmi.exceptions.NoSuchContextException;
-
-import java.rmi.Remote;
-import java.rmi.RemoteException;
 
 /**
  * This class defines the Open-Xchange API Version 2 for creating and
@@ -214,6 +213,18 @@ public interface OXUserInterface extends Remote {
      */
     public UserModuleAccess getContextAdminUserModuleAccess(Context ctx, Credentials auth)  throws RemoteException, StorageException,InvalidCredentialsException, NoSuchContextException,InvalidDataException, DatabaseUpdateException;
 
+    /**
+     * Returns the Context admin {@link User} object
+     * 
+     * @param ctx
+     * @param auth
+     * @return
+     * @throws RemoteException
+     * @throws InvalidCredentialsException
+     * @throws StorageException
+     * @throws InvalidDataException
+     */
+    public User getContextAdmin(Context ctx, Credentials auth) throws RemoteException, InvalidCredentialsException, StorageException, InvalidDataException;
     
     /**
      * Manipulate user data within the given context.
@@ -388,7 +399,21 @@ public interface OXUserInterface extends Remote {
      */
     public void changeModuleAccess(final Context ctx, final User user, final String access_combination_name, final Credentials auth) throws RemoteException, StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, DatabaseUpdateException, NoSuchUserException;
 
-    
+    /**
+     * This method changes module Permissions for all (!) users in all (!) contexts. This can be filtered by already existing access combinations.
+     * If no filter is given, all users are changed.
+     * 
+     * @param filter The call affects only users with exactly this access combination. This is either a String representing a defined module access combination or an Integer (masked as String) for direct definitions. null for no filter.
+     * @param addAccess Access rights to be added
+     * @param removeAccess Access rights to be removed
+     * @param auth Credentials for authenticating against server. Must be the master Admin.
+     * @throws InvalidCredentialsException When the supplied credentials were not correct or invalid.
+     * @throws StorageException When an error in the subsystems occured.
+     * @throws InvalidDataException If the data sent within the method contained invalid data.
+     * @throws RemoteException General RMI Exception
+     */
+    public void changeModuleAccessGlobal(String filter, UserModuleAccess addAccess, UserModuleAccess removeAccess, Credentials auth) throws RemoteException, InvalidCredentialsException, StorageException, InvalidDataException;
+
     /**
      * Retrieve user objects for a range of users by username or id.
      * 
@@ -447,6 +472,9 @@ public interface OXUserInterface extends Remote {
 
     /**
      * Retrieve all users for a given context.
+     * The search pattern is directly transformed into a SQL LIKE string comparison, where<br>
+     * a * is transformed into a %<br>
+     * a % and a _ must be escaped by a \ (e.g. if you want to search for _doe, use the pattern \_doe
      * 
      * @param ctx
      *            Context object.
@@ -465,6 +493,29 @@ public interface OXUserInterface extends Remote {
      */
     public User[] list(final Context ctx, final String search_pattern, final Credentials auth) throws RemoteException, StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, DatabaseUpdateException;
 
+    /**
+     * Retrieve all users for a given context.
+     * The search pattern is directly transformed into a SQL LIKE string comparison, where<br>
+     * a * is transformed into a %<br>
+     * a % and a _ must be escaped by a \ (e.g. if you want to search for _doe, use the pattern \_doe
+     * 
+     * @param ctx
+     *            Context object.
+     * @param search_pattern
+     *            A pattern to search for
+     * @param auth
+     *            Credentials for authenticating against server.
+     * @return User[] with currently ONLY id set in each User.
+     * 
+     * @throws RemoteException
+     * @throws StorageException
+     * @throws InvalidCredentialsException
+     * @throws NoSuchContextException
+     * @throws InvalidDataException
+     * @throws DatabaseUpdateException
+     */
+    public User[] listCaseInsensitive(final Context ctx, final String search_pattern, final Credentials auth) throws RemoteException, StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, DatabaseUpdateException;
+
     /**
      * Retrieve all users for a given context. The same as calling list with a search_pattern of "*"
      * 
@@ -483,5 +534,18 @@ public interface OXUserInterface extends Remote {
      */
     public User[] listAll(final Context ctx, final Credentials auth) throws RemoteException, StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, DatabaseUpdateException;
 
-    
+    /**
+     * Check whether the given user exists. Either users id or name must be set.
+     * 
+     * @param ctx
+     * @param user
+     * @return
+     * @throws RemoteException
+     * @throws InvalidDataException 
+     * @throws InvalidCredentialsException 
+     * @throws StorageException 
+     * @throws DatabaseUpdateException 
+     * @throws NoSuchContextException 
+     */
+    public boolean exists(final Context ctx, final User user, final Credentials auth) throws RemoteException, InvalidDataException, InvalidCredentialsException, StorageException, DatabaseUpdateException, NoSuchContextException;
 }
