@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,7 +51,9 @@ package com.openexchange.admin.rmi.dataobjects;
 
 import java.lang.reflect.Field;
 import java.util.Collection;
+import java.util.HashMap;
 import java.util.HashSet;
+import java.util.Map;
 
 /**
  * Class representing a context.
@@ -113,6 +115,10 @@ public class Context extends ExtendableDataObject implements NameAndIdObject {
     
     private HashSet<String> login_mappings;
     
+    private Map<String, Map<String, String>> userAttributes = null;
+
+    private boolean userAttribtuesset;
+
     private boolean listrun;
 
     public Context() {
@@ -126,7 +132,7 @@ public class Context extends ExtendableDataObject implements NameAndIdObject {
     public Context(final Integer id) {
         super();
         init();
-        this.id = id;
+        setId(id);
     }
 
     /**
@@ -309,7 +315,12 @@ public class Context extends ExtendableDataObject implements NameAndIdObject {
         this.filestore_nameset = true;
     }
 
-    public final String toString() {
+    /*
+     * DO NOT make this final because others might want to extend this
+     * (non-Javadoc)
+     * @see com.openexchange.admin.rmi.dataobjects.ExtendableDataObject#toString()
+     */
+    public String toString() {
         StringBuilder ret = new StringBuilder();
         ret.append("[ \n");
         for (final Field f : this.getClass().getDeclaredFields()) {
@@ -457,6 +468,52 @@ public class Context extends ExtendableDataObject implements NameAndIdObject {
     public boolean isWriteDatabaseset() {
         return writeDatabaseset;
     }
+    
+    /**
+     * Sets a generic user attribute
+     */
+    public void setUserAttribute(String namespace, String name, String value) {
+        getNamespace(namespace).put(name, value);
+        userAttribtuesset = true;
+    }
+    
+    /**
+     * Read a generic user attribute
+     */
+    public String getUserAttribute(String namespace, String name) {
+        return getNamespace(namespace).get(name);
+    }
+    
+    public Map<String, Map<String, String>> getUserAttributes() {
+        if(userAttributes == null) {
+            userAttributes = new HashMap<String, Map<String, String>>();
+        }
+        return userAttributes;
+    }
+    
+    public void setUserAttributes(Map<String, Map<String, String>> userAttributes) {
+        this.userAttribtuesset = true;
+        this.userAttributes = userAttributes;
+    }
+    
+    public Map<String, String> getNamespace(String namespace) {
+        if(userAttributes == null) {
+            userAttributes = new HashMap<String, Map<String, String>>();
+        }
+        Map<String, String> ns = userAttributes.get(namespace);
+        if(ns == null) {
+            ns = new HashMap<String, String>();
+            userAttributes.put(namespace, ns);
+        }
+        return ns;
+    }
+    
+    /**
+     * Used to check if the user attributes have been modified
+     */
+    public boolean isUserAttributesset() {
+        return userAttribtuesset;
+    }
 
     /* (non-Javadoc)
      * @see java.lang.Object#hashCode()
