@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -135,7 +135,7 @@ public class Database extends EnforceableDataObject implements NameAndIdObject {
     public Database(final int id) {
         super();
         init();
-        this.id = id;
+        this.id = Integer.valueOf(id);
     }
 
     /**
@@ -145,7 +145,7 @@ public class Database extends EnforceableDataObject implements NameAndIdObject {
     public Database(final int id, final String schema) {
         super();
         init();
-        this.id = id;
+        this.id = Integer.valueOf(id);
         this.scheme = schema;
     }
 
@@ -163,7 +163,7 @@ public class Database extends EnforceableDataObject implements NameAndIdObject {
         this.password = password;
         this.driver = driver;
         this.url = url;
-        this.id = id;
+        this.id = Integer.valueOf(id);
         this.name = displayname;
     }
 
