@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,6 +51,7 @@ package com.openexchange.admin.rmi.dataobjects;
 
 import java.io.Serializable;
 import java.lang.reflect.Field;
+import com.openexchange.groupware.userconfiguration.UserConfiguration;
 
 /**
  * Object for setting/getting access informations to the different ox modules
@@ -124,9 +125,13 @@ public class UserModuleAccess implements Serializable {
 
     private boolean USM = true;
 
+    private boolean OLOX20 = true;
+
     private boolean GlobalAddressBookDisabled = false;
 
-    private boolean PublicFolderEditable = true;
+    private boolean PublicFolderEditable = false;
+
+    private boolean deniedPortal;
 
     /**
      * Creates a new instance of UserModuleAccess
@@ -135,7 +140,6 @@ public class UserModuleAccess implements Serializable {
         super();
     }
 
-
     /**
      * Enable all modules
      */
@@ -169,6 +173,7 @@ public class UserModuleAccess implements Serializable {
         this.USM = true;
         this.GlobalAddressBookDisabled = false;
         this.PublicFolderEditable = true;
+        this.OLOX20 = true;
     }
 
     /**
@@ -204,6 +209,7 @@ public class UserModuleAccess implements Serializable {
         this.USM = false;
         this.GlobalAddressBookDisabled = true;
         this.PublicFolderEditable = false;
+        this.OLOX20 = false;
     }
 
     public boolean getEditGroup() {
@@ -524,6 +530,22 @@ public class UserModuleAccess implements Serializable {
         this.USM = val;
     }
 
+    public boolean isOLOX20() {
+        return OLOX20;
+    }
+
+    public void setOLOX20(final boolean val) {
+        this.OLOX20 = val;
+    }
+
+    public void setDeniedPortal(final boolean val) {
+        this.deniedPortal = val;
+    }
+
+    public boolean isDeniedPortal() {
+        return deniedPortal;
+    }
+
     public boolean isGlobalAddressBookDisabled() {
         return GlobalAddressBookDisabled;
     }
@@ -670,5 +692,96 @@ public class UserModuleAccess implements Serializable {
         }
         return true;
     }
+    
+    public int getPermissionBits() {
+        int retval = 0;
+        
+        if (isActiveSync()) {
+            retval |= UserConfiguration.ACTIVE_SYNC;
+        }
+        if (getCalendar()) {
+            retval |= UserConfiguration.CALENDAR;
+        }
+        if (isCollectEmailAddresses()) {
+            retval |= UserConfiguration.COLLECT_EMAIL_ADDRESSES;
+        }
+        if (getContacts()) {
+            retval |= UserConfiguration.CONTACTS;
+        }
+        if (getDelegateTask()) {
+            retval |= UserConfiguration.DELEGATE_TASKS;
+        }
+        if (getEditGroup()) {
+            retval |= UserConfiguration.EDIT_GROUP;
+        }
+        if (getEditPassword()) {
+            retval |= UserConfiguration.EDIT_PASSWORD;
+        }
+        if (getEditPublicFolders()) {
+            retval |= UserConfiguration.EDIT_PUBLIC_FOLDERS;
+        }
+        if (getEditResource()) {
+            retval |= UserConfiguration.EDIT_RESOURCE;
+        }
+        if (getForum()) {
+            retval |= UserConfiguration.FORUM;
+        }
+        if (getIcal()) {
+            retval |= UserConfiguration.ICAL;
+        }
+        if (getInfostore()) {
+            retval |= UserConfiguration.INFOSTORE;
+        }
+        if (getSyncml()) {
+            retval |= UserConfiguration.MOBILITY;
+        }
+        if (isMultipleMailAccounts()) {
+            retval |= UserConfiguration.MULTIPLE_MAIL_ACCOUNTS;
+        }
+        if (isOLOX20()) {
+            retval |= UserConfiguration.OLOX20;
+        }
+        if (getPinboardWrite()) {
+            retval |= UserConfiguration.PINBOARD_WRITE_ACCESS;
+        }
+        if (getProjects()) {
+            retval |= UserConfiguration.PROJECTS;
+        }
+        if (isPublication()) {
+            retval |= UserConfiguration.PUBLICATION;
+        }
+        if (getReadCreateSharedFolders()) {
+            retval |= UserConfiguration.READ_CREATE_SHARED_FOLDERS;
+        }
+        if (getRssBookmarks()) {
+            retval |= UserConfiguration.RSS_BOOKMARKS;
+        }
+        if (getRssPortal()) {
+            retval |= UserConfiguration.RSS_PORTAL;
+        }
+        if (isSubscription()) {
+            retval |= UserConfiguration.SUBSCRIPTION;
+        }
+        if (getTasks()) {
+            retval |= UserConfiguration.TASKS;
+        }
+        if (isUSM()) {
+            retval |= UserConfiguration.USM;
+        }
+        if (getVcard()) {
+            retval |= UserConfiguration.VCARD;
+        }
+        if (getWebdav()) {
+            retval |= UserConfiguration.WEBDAV;
+        }
+        if (getWebdavXml()) {
+            retval |= UserConfiguration.WEBDAV_XML;
+        }
+        if (getWebmail()) {
+            retval |= UserConfiguration.WEBMAIL;
+        }
+        
+        return retval;
+    }
 
 }
