@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -48,6 +48,7 @@
  */
 package com.openexchange.admin.rmi.impl;
 
+import static com.openexchange.java.Autoboxing.I2i;
 import java.rmi.RemoteException;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -83,6 +84,7 @@ import com.openexchange.admin.tools.PropertyHandler;
 import com.openexchange.caching.Cache;
 import com.openexchange.caching.CacheException;
 import com.openexchange.caching.CacheService;
+import com.openexchange.java.Autoboxing;
 
 /**
  * Implementation for the RMI interface of group
@@ -129,11 +131,12 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
     }
 
     public void addMember(final Context ctx, final Group grp,
-            final User[] members, final Credentials auth)
+            final User[] members, Credentials auth)
             throws RemoteException, StorageException,
             InvalidCredentialsException, NoSuchContextException,
             InvalidDataException, DatabaseUpdateException, NoSuchUserException,
             NoSuchGroupException {
+        auth = auth == null ? new Credentials("","") : auth;
         try {
             doNullCheck(grp, members);
         } catch (final InvalidDataException e3) {
@@ -196,7 +199,8 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
         // END OF JCS
     }
 
-    public void change(final Context ctx, final Group grp, final Credentials auth) throws RemoteException, StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, DatabaseUpdateException, NoSuchGroupException, NoSuchUserException {
+    public void change(final Context ctx, final Group grp, Credentials auth) throws RemoteException, StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, DatabaseUpdateException, NoSuchGroupException, NoSuchUserException {
+        auth = auth == null ? new Credentials("","") : auth;
         try {
             doNullCheck(grp);
         } catch (final InvalidDataException e3) {
@@ -331,9 +335,10 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
     }
 
     public Group create(final Context ctx, final Group grp,
-            final Credentials auth) throws RemoteException, StorageException,
+            Credentials auth) throws RemoteException, StorageException,
             InvalidCredentialsException, NoSuchContextException,
             InvalidDataException, DatabaseUpdateException, NoSuchUserException {
+        auth = auth == null ? new Credentials("","") : auth;
         try {
             doNullCheck(grp);
         } catch (final InvalidDataException e3) {
@@ -375,12 +380,7 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
 
             // if members sent, check exist
             if (grp.getMembers() != null && grp.getMembers().length > 0) {
-                final Integer[] mems = grp.getMembers();
-                final int[] tmp_mems = new int[mems.length];
-                for (int i = 0; i < mems.length; i++) {
-                    tmp_mems[i] = mems[i];
-                }
-                if (!tool.existsUser(ctx, tmp_mems)) {
+                if (!tool.existsUser(ctx, I2i(grp.getMembers()))) {
                     throw new NoSuchUserException("No such user");
                 }
             }
@@ -499,10 +499,11 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
     }
 
     public void delete(final Context ctx, final Group grp,
-            final Credentials auth) throws RemoteException,
+            Credentials auth) throws RemoteException,
             InvalidCredentialsException, NoSuchContextException,
             StorageException, InvalidDataException, DatabaseUpdateException,
             NoSuchGroupException {
+        auth = auth == null ? new Credentials("","") : auth;
         try {
             doNullCheck(grp);
         } catch (final InvalidDataException e3) {
@@ -514,9 +515,10 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
     }
 
     public void delete(final Context ctx, final Group[] grp,
-            final Credentials auth) throws RemoteException, StorageException,
+            Credentials auth) throws RemoteException, StorageException,
             InvalidCredentialsException, NoSuchContextException,
             InvalidDataException, DatabaseUpdateException, NoSuchGroupException {
+        auth = auth == null ? new Credentials("","") : auth;
         try {
             doNullCheck((Object[]) grp);
         } catch (final InvalidDataException e3) {
@@ -643,9 +645,10 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
     }
 
     public Group getData(final Context ctx, final Group grp,
-            final Credentials auth) throws RemoteException, StorageException,
+            Credentials auth) throws RemoteException, StorageException,
             InvalidCredentialsException, NoSuchContextException,
             InvalidDataException, DatabaseUpdateException, NoSuchGroupException {
+        auth = auth == null ? new Credentials("","") : auth;
         try {
             doNullCheck(grp);
         } catch (final InvalidDataException e3) {
@@ -705,9 +708,10 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
     }
 
     public Group[] getData(final Context ctx, final Group[] groups,
-            final Credentials auth) throws RemoteException, StorageException,
+            Credentials auth) throws RemoteException, StorageException,
             InvalidCredentialsException, NoSuchContextException,
             InvalidDataException, NoSuchGroupException, DatabaseUpdateException {
+        auth = auth == null ? new Credentials("","") : auth;
         try {
             doNullCheck((Object[]) groups);
         } catch (final InvalidDataException e3) {
@@ -791,10 +795,11 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
         return retval.toArray(new Group[retval.size()]);
     }
 
-    public Group getDefaultGroup(final Context ctx, final Credentials auth)
+    public Group getDefaultGroup(final Context ctx, Credentials auth)
             throws RemoteException, StorageException,
             InvalidCredentialsException, NoSuchContextException,
             InvalidDataException, DatabaseUpdateException {
+        auth = auth == null ? new Credentials("","") : auth;
         basicauth.doAuthentication(auth, ctx);
         if (log.isDebugEnabled()) {
             log.debug(ctx + " - " + auth);
@@ -811,9 +816,10 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
     }
 
     public User[] getMembers(final Context ctx, final Group grp,
-            final Credentials auth) throws RemoteException, StorageException,
+            Credentials auth) throws RemoteException, StorageException,
             InvalidCredentialsException, NoSuchContextException,
             InvalidDataException, DatabaseUpdateException, NoSuchGroupException {
+        auth = auth == null ? new Credentials("","") : auth;
         try {
             doNullCheck(grp);
         } catch (final InvalidDataException e3) {
@@ -846,9 +852,10 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
     }
 
     public Group[] list(final Context ctx, final String pattern,
-            final Credentials auth) throws RemoteException, StorageException,
+            Credentials auth) throws RemoteException, StorageException,
             InvalidCredentialsException, NoSuchContextException,
             InvalidDataException, DatabaseUpdateException {
+        auth = auth == null ? new Credentials("","") : auth;
         try {
             doNullCheck(pattern);
         } catch (final InvalidDataException e3) {
@@ -880,10 +887,11 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
     }
 
     public Group[] listGroupsForUser(final Context ctx, final User usr,
-            final Credentials auth) throws RemoteException,
+            Credentials auth) throws RemoteException,
             InvalidCredentialsException, NoSuchContextException,
             StorageException, InvalidDataException, DatabaseUpdateException,
             NoSuchUserException {
+        auth = auth == null ? new Credentials("","") : auth;
         try {
             doNullCheck(usr);
         } catch (final InvalidDataException e3) {
@@ -913,11 +921,12 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
     }
 
     public void removeMember(final Context ctx, final Group grp,
-            final User[] members, final Credentials auth)
+            final User[] members, Credentials auth)
             throws RemoteException, StorageException,
             InvalidCredentialsException, NoSuchContextException,
             InvalidDataException, DatabaseUpdateException,
             NoSuchGroupException, NoSuchUserException {
+        auth = auth == null ? new Credentials("","") : auth;
         try {
             doNullCheck(grp, members);
         } catch (final InvalidDataException e3) {
