@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,15 +49,19 @@
 
 package com.openexchange.admin.rmi.impl;
 
+import static com.openexchange.java.Autoboxing.i;
 import java.io.BufferedReader;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStreamReader;
 import java.io.UnsupportedEncodingException;
+import java.rmi.RemoteException;
 import java.security.NoSuchAlgorithmException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashSet;
+import java.util.Set;
+import javax.mail.internet.IDNA;
 import org.apache.commons.io.FileUtils;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
@@ -91,6 +95,7 @@ import com.openexchange.caching.Cache;
 import com.openexchange.caching.CacheException;
 import com.openexchange.caching.CacheKey;
 import com.openexchange.caching.CacheService;
+import com.openexchange.groupware.container.FolderObject;
 import com.openexchange.groupware.contexts.impl.ContextImpl;
 import com.openexchange.groupware.userconfiguration.UserConfigurationException;
 import com.openexchange.groupware.userconfiguration.UserConfigurationStorage;
@@ -137,7 +142,8 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
         return this.cache.getProperties().getUserProp(AdminProperties.User.USERNAME_CHANGEABLE, false);
     }
 
-    public void change(final Context ctx, final User usrdata, final Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException,InvalidDataException, DatabaseUpdateException, NoSuchUserException {
+    public void change(final Context ctx, final User usrdata, Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException,InvalidDataException, DatabaseUpdateException, NoSuchUserException {
+        auth = auth == null ? new Credentials("","") : auth;
         try {
             doNullCheck(usrdata);
         } catch (final InvalidDataException e2) {
@@ -183,6 +189,9 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
                 log.error(noSuchUserException.getMessage(), noSuchUserException);
                 throw noSuchUserException;
             }
+            if (tool.existsDisplayName(ctx, usrdata, i(usrdata.getId()))) {
+                throw new InvalidDataException("The displayname is already used");
+            }
             final User[] dbuser = oxu.getData(ctx, new User[] { usrdata });
 
             checkChangeUserData(ctx, usrdata, dbuser[0], this.prop);
@@ -249,29 +258,31 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
             }
             ClientAdminThread.cache.setAdminCredentials(ctx,mech,cauth);
         }
-        final CacheService cacheService = AdminDaemon.getService(SYMBOLIC_NAME_CACHE, NAME_OXCACHE, context,
-				CacheService.class);
+        final CacheService cacheService = AdminDaemon.getService(SYMBOLIC_NAME_CACHE, NAME_OXCACHE, context, CacheService.class);
 		if (null != cacheService) {
 	        try {
 	        	final CacheKey key = cacheService.newCacheKey(ctx.getId().intValue(), userid);
-	        	Cache cache = cacheService.getCache("User");
-	        	cache.remove(key);
+	        	Cache jcs = cacheService.getCache("User");
+	        	jcs.remove(key);
 	        	
-	        	cache = cacheService.getCache("UserConfiguration");
-	        	cache.remove(key);
+	        	jcs = cacheService.getCache("UserConfiguration");
+	        	jcs.remove(key);
 	        	
-	        	cache = cacheService.getCache("UserSettingMail");
-	        	cache.remove(key);
+	        	jcs = cacheService.getCache("UserSettingMail");
+	        	jcs.remove(key);
+
+	        	jcs = cacheService.getCache("MailAccount");
+	        	jcs.invalidateGroup(ctx.getId().toString());
 	        } catch (final CacheException e) {
 	            log.error(e.getMessage(), e);
 	        } finally {
 	        	AdminDaemon.ungetService(SYMBOLIC_NAME_CACHE, NAME_OXCACHE, context);
 	        }
         }
-
     }
 
-    public void changeModuleAccess(final Context ctx, final User user, final UserModuleAccess moduleAccess, final Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException,InvalidDataException, DatabaseUpdateException, NoSuchUserException {
+    public void changeModuleAccess(final Context ctx, final User user, final UserModuleAccess moduleAccess, Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException,InvalidDataException, DatabaseUpdateException, NoSuchUserException {
+        auth = auth == null ? new Credentials("","") : auth;
         try {
             doNullCheck(user,moduleAccess);
         } catch (final InvalidDataException e1) {
@@ -322,9 +333,10 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
         // END OF JCS
     }
 
-    public void changeModuleAccess(final Context ctx, final User user,final String access_combination_name, final Credentials auth)
+    public void changeModuleAccess(final Context ctx, final User user,final String access_combination_name, Credentials auth)
 			throws StorageException,InvalidCredentialsException, NoSuchContextException,InvalidDataException, DatabaseUpdateException, NoSuchUserException {
-		
+
+        auth = auth == null ? new Credentials("","") : auth;
     	try {
             doNullCheck(user,access_combination_name);
             if (access_combination_name.trim().length() == 0) {
@@ -356,6 +368,10 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
             	// throw error!
             	throw new InvalidDataException("No such access combination name \""+access_combination_name.trim()+"\"");
             }
+            if (access.isPublicFolderEditable() && user_id != tool.getAdminForContext(ctx)) {
+                // publicFolderEditable can only be applied to the context administrator.
+                access.setPublicFolderEditable(false);
+            }
             oxu.changeModuleAccess(ctx, user_id, access);
         } catch (final StorageException e) {
             log.error(e.getMessage(), e);
@@ -392,18 +408,19 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
 		
 	}
 
-    public User create(Context ctx, User usr, UserModuleAccess access, Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, DatabaseUpdateException {
+    public User create(final Context ctx, final User usr, final UserModuleAccess access, final Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, DatabaseUpdateException {
         // Call common create method directly because we already have out access module
     	return createUserCommon(ctx, usr, access, auth);
     }
 
-    public User create(Context ctx, User usrdata, String access_combination_name, Credentials auth) throws StorageException,InvalidCredentialsException, NoSuchContextException,InvalidDataException, DatabaseUpdateException {
+    public User create(final Context ctx, final User usrdata, final String access_combination_name, Credentials auth) throws StorageException,InvalidCredentialsException, NoSuchContextException,InvalidDataException, DatabaseUpdateException {
 
     	/*
 		 * Resolve the access rights by the specified combination name. If
 		 * combination name does not exists, throw error as it is described in
 		 * the spec!
 		 */
+        auth = auth == null ? new Credentials("","") : auth;
 
     	
     	try {
@@ -431,19 +448,24 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
         	throw new InvalidDataException("No such access combination name \""+access_combination_name.trim()+"\"");
         }
 
+        if (access.isPublicFolderEditable()) {
+            // publicFolderEditable can only be applied to the context administrator.
+            access.setPublicFolderEditable(false);
+        }
         // Call main create user method with resolved access rights
     	return createUserCommon(ctx, usrdata, access, auth);
     }
 
 
-    public User create(Context ctx, User usrdata, Credentials auth)	throws StorageException,InvalidCredentialsException, NoSuchContextException,InvalidDataException, DatabaseUpdateException {
+    public User create(final Context ctx, final User usrdata, Credentials auth)	throws StorageException,InvalidCredentialsException, NoSuchContextException,InvalidDataException, DatabaseUpdateException {
 
 		/*
 		 * Resolve current access rights from the specified context (admin) as
 		 * it is described in the spec and then call the main create user method
 		 * with the access rights!
 		 */
-    	
+        auth = auth == null ? new Credentials("","") : auth;
+
     	try {
 			doNullCheck(usrdata);			
 		} catch (final InvalidDataException e3) {
@@ -463,11 +485,24 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
 		
 		final int admin_id = tool.getAdminForContext(ctx);
 		final UserModuleAccess access = oxu.getModuleAccess(ctx, admin_id);
-		
+
+		if (access.isPublicFolderEditable()) {
+            // publicFolderEditable can only be applied to the context administrator.
+            access.setPublicFolderEditable(false);
+        }
+
     	return createUserCommon(ctx, usrdata, access, auth);
     }
+
+    public User getContextAdmin(final Context ctx, Credentials auth) throws InvalidCredentialsException, StorageException, InvalidDataException {
+        auth = auth == null ? new Credentials("","") : auth;
+        
+        basicauth.doAuthentication(auth, ctx);
+        return (oxu.getData(ctx, new User[]{ new User(tool.getAdminForContext(ctx))} ))[0];
+    }
     
-    public UserModuleAccess getContextAdminUserModuleAccess(Context ctx, Credentials auth)  throws StorageException,InvalidCredentialsException, NoSuchContextException,InvalidDataException, DatabaseUpdateException {
+    public UserModuleAccess getContextAdminUserModuleAccess(final Context ctx, Credentials auth)  throws StorageException,InvalidCredentialsException, NoSuchContextException,InvalidDataException, DatabaseUpdateException {
+        auth = auth == null ? new Credentials("","") : auth;
         basicauth.doAuthentication(auth, ctx);
         
         /*
@@ -482,7 +517,7 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
     /*
      * Main method to create a user. Which all inner create methods MUST use after resolving the access rights!
      */
-    private User createUserCommon(Context ctx, User usr, UserModuleAccess access, Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, DatabaseUpdateException {
+    private User createUserCommon(final Context ctx, final User usr, final UserModuleAccess access, final Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, DatabaseUpdateException {
     	
     	try {
             doNullCheck(usr,access);
@@ -588,14 +623,37 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
                 }
             }
         }
+        // The mail account cache caches resolved imap logins or primary addresses. Creating or changing a user needs the invalidation of
+        // that cached data.
+        final CacheService cacheService = AdminDaemon.getService(SYMBOLIC_NAME_CACHE, NAME_OXCACHE, context, CacheService.class);
+        if (null != cacheService) {
+            try {
+                final Cache mailAccountCache = cacheService.getCache("MailAccount");
+                mailAccountCache.invalidateGroup(ctx.getId().toString());
+
+                final Cache globalFolderCache = cacheService.getCache("GlobalFolderCache");
+                CacheKey cacheKey = cacheService.newCacheKey(1, "0", Integer.toString(FolderObject.SYSTEM_LDAP_FOLDER_ID));
+                globalFolderCache.removeFromGroup(cacheKey, ctx.getId().toString());
+
+                final Cache folderCache = cacheService.getCache("OXFolderCache");
+                cacheKey = cacheService.newCacheKey(ctx.getId().intValue(), FolderObject.SYSTEM_LDAP_FOLDER_ID);
+                folderCache.remove(cacheKey);
+            } catch (final CacheException e) {
+                log.error(e.getMessage(), e);
+            } finally {
+                AdminDaemon.ungetService(SYMBOLIC_NAME_CACHE, NAME_OXCACHE, context);
+            }
+        }
         return usr;
     }
 
-    public void delete(Context ctx, User user, Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, DatabaseUpdateException, NoSuchUserException {
+    public void delete(final Context ctx, final User user, final Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, DatabaseUpdateException, NoSuchUserException {
         delete(ctx, new User[]{user}, auth);
     }
 
-    public void delete(Context ctx, User[] users, Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException,InvalidDataException, DatabaseUpdateException, NoSuchUserException {
+    public void delete(final Context ctx, final User[] users, Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException,InvalidDataException, DatabaseUpdateException, NoSuchUserException {
+        auth = auth == null ? new Credentials("","") : auth;
+
         try {
             doNullCheck((Object[])users);
         } catch (final InvalidDataException e1) {
@@ -624,7 +682,12 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
                 log.error("No such user(s) " + Arrays.toString(users) + " in context " + ctx.getId(), noSuchUserException);
                 throw noSuchUserException;
             }
+            final Set<Integer> dubCheck = new HashSet<Integer>();
             for (final User user : users) {
+                if (dubCheck.contains(user.getId())) {
+                    throw new InvalidDataException("User " + user.getId() + " is contained multiple times in delete request.");
+                }
+                dubCheck.add(user.getId());
                 if (tool.isContextAdmin(ctx, user.getId().intValue())) {
                     throw new InvalidDataException("Admin delete not supported");
                 }
@@ -702,18 +765,17 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
 
         oxu.delete(ctx, users);
 
-
         // JCS
-        final CacheService cacheService = AdminDaemon.getService(SYMBOLIC_NAME_CACHE, NAME_OXCACHE, context,
-				CacheService.class);
+        final CacheService cacheService = AdminDaemon.getService(SYMBOLIC_NAME_CACHE, NAME_OXCACHE, context, CacheService.class);
 		if (null != cacheService) {
 			try {
 				final Cache usercCache = cacheService.getCache("User");
 				final Cache ucCache = cacheService.getCache("UserConfiguration");
 				final Cache usmCache = cacheService.getCache("UserSettingMail");
 				for (final User user : users) {
-					final CacheKey key = cacheService.newCacheKey(ctx.getId().intValue(), user.getId());
+					final CacheKey key = cacheService.newCacheKey(i(ctx.getId()), user.getId());
 					usercCache.remove(key);
+					usercCache.remove(cacheService.newCacheKey(i(ctx.getId()), user.getName()));
 					ucCache.remove(key);
 					usmCache.remove(key);
 					try {
@@ -742,11 +804,12 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
         }
     }
 
-    public User getData(Context ctx, User user, Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, NoSuchUserException, DatabaseUpdateException {
+    public User getData(final Context ctx, final User user, final Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, NoSuchUserException, DatabaseUpdateException {
         return getData(ctx, new User[]{user}, auth)[0];
     }
 
-    public User[] getData(Context ctx, User[] users, Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, NoSuchUserException, DatabaseUpdateException {
+    public User[] getData(final Context ctx, final User[] users, Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, NoSuchUserException, DatabaseUpdateException {
+        auth = auth == null ? new Credentials("","") : auth;
         try {
             doNullCheck((Object[])users);
         } catch (final InvalidDataException e1) {
@@ -874,7 +937,8 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
         return retusers;
     }
 
-    public UserModuleAccess getModuleAccess(Context ctx, User user, Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException,InvalidDataException, DatabaseUpdateException, NoSuchUserException {
+    public UserModuleAccess getModuleAccess(final Context ctx, final User user, Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException,InvalidDataException, DatabaseUpdateException, NoSuchUserException {
+        auth = auth == null ? new Credentials("","") : auth;
         try {
             doNullCheck(user);
         } catch (final InvalidDataException e) {
@@ -915,8 +979,9 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
         }
     }
 
-    public String getAccessCombinationName(Context ctx, User user, Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, DatabaseUpdateException, NoSuchUserException {
-    	
+    public String getAccessCombinationName(final Context ctx, final User user, Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, DatabaseUpdateException, NoSuchUserException {
+        auth = auth == null ? new Credentials("","") : auth;
+
     	try {
             doNullCheck(user);
         } catch (final InvalidDataException e) {
@@ -966,28 +1031,8 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
 
 
 
-    public boolean isContextAdmin(Context ctx, User user, Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, NoSuchUserException, DatabaseUpdateException {
-        try {
-            doNullCheck(user);
-        } catch (final InvalidDataException e1) {
-            log.error("One of the given arguments is null", e1);
-            throw e1;
-        }
-
-        basicauth.doUserAuthentication(auth,ctx);
-
-        checkContextAndSchema(ctx);
-
-        if (user.getId()!=null && !tool.existsUser(ctx, user.getId().intValue())) {
-            final NoSuchUserException noSuchUserException = new NoSuchUserException("No such user " + user.getId().intValue() + " in context " + ctx.getId());
-            log.error(noSuchUserException.getMessage(), noSuchUserException);
-            throw noSuchUserException;
-        }
-
-        return tool.isContextAdmin(ctx, user.getId().intValue());
-    }
-
-    public User[] list(Context ctx, String search_pattern, Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, DatabaseUpdateException {
+    public User[] list(final Context ctx, final String search_pattern, Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, DatabaseUpdateException {
+        auth = auth == null ? new Credentials("","") : auth;
         try {
             doNullCheck(ctx,search_pattern);
         } catch (final InvalidDataException e1) {
@@ -1008,7 +1053,29 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
         return retval;
     }
 
-    public User[] listAll(Context ctx, Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, DatabaseUpdateException {
+    public User[] listCaseInsensitive(final Context ctx, final String search_pattern, Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, DatabaseUpdateException {
+        auth = auth == null ? new Credentials("","") : auth;
+        try {
+            doNullCheck(ctx,search_pattern);
+        } catch (final InvalidDataException e1) {
+            log.error("One of the given arguments for list is null", e1);
+            throw e1;
+        }
+
+        if (log.isDebugEnabled()) {
+            log.debug(ctx + " - " + auth);
+        }
+
+        basicauth.doAuthentication(auth,ctx);
+
+        checkContextAndSchema(ctx);
+
+        final User[] retval =  oxu.listCaseInsensitive(ctx, search_pattern);
+
+        return retval;
+    }
+
+    public User[] listAll(final Context ctx, final Credentials auth) throws StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, DatabaseUpdateException {
         return list(ctx, "*", auth);
     }
 
@@ -1036,7 +1103,7 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
      * @throws StorageException
      * @throws InvalidDataException
      */
-    private void checkChangeUserData(Context ctx, User newuser, User dbuser, PropertyHandler prop) throws StorageException, InvalidDataException {
+    private void checkChangeUserData(final Context ctx, final User newuser, final User dbuser, final PropertyHandler prop) throws StorageException, InvalidDataException {
         if (newuser.getName() != null) {
             if (usernameIsChangeable()) {
                 if (prop.getUserProp(AdminProperties.User.CHECK_NOT_ALLOWED_CHARS, true)) {
@@ -1085,6 +1152,15 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
             if (useraliases == null) {
                 useraliases = dbuser.getAliases();
             }
+            if (null != useraliases) {
+                final HashSet<String> tmp = new HashSet<String>(useraliases.size());
+                for (final String email : useraliases) {
+                    tmp.add(IDNA.toIDN(email));
+                }
+                useraliases = tmp;
+            } else {
+                useraliases = new HashSet<String>(1);
+            }
 
             final String defaultSenderAddress = newuser.getDefaultSenderAddress();
             final String primaryEmail = newuser.getPrimaryEmail();
@@ -1098,23 +1174,26 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
             String check_email1;
             String check_default_sender_address;
             if (primaryEmail != null) {
-                check_primary_mail = primaryEmail;
+                check_primary_mail = IDNA.toIDN(primaryEmail);
                 if (!primaryEmail.equals(dbuser.getPrimaryEmail())) {
                     tool.primaryMailExists(ctx, primaryEmail);
                 }
             } else {
-                check_primary_mail = dbuser.getPrimaryEmail();
+                final String email = dbuser.getPrimaryEmail();
+                check_primary_mail = email == null ? email : IDNA.toIDN(email);
             }
 
             if (email1 != null) {
-                check_email1 = email1;
+                check_email1 = IDNA.toIDN(email1);
             } else {
-                check_email1 = dbuser.getEmail1();
+                final String s = dbuser.getEmail1();
+                check_email1 = s == null ? s : IDNA.toIDN(s);
             }
             if (defaultSenderAddress != null) {
-                check_default_sender_address = defaultSenderAddress;
+                check_default_sender_address = IDNA.toIDN(defaultSenderAddress);
             } else {
-                check_default_sender_address = dbuser.getDefaultSenderAddress();
+                final String s = dbuser.getDefaultSenderAddress();
+                check_default_sender_address = s == null ? s : IDNA.toIDN(s);
             }
 
             final boolean found_primary_mail = useraliases.contains(check_primary_mail);
@@ -1140,6 +1219,18 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
         if (null == ctx.getId()) {
             throw new InvalidDataException("Context invalid");
         }
+        /*-
+         * Check a context existence is considered as a security flaw
+         * 
+        try {
+            if (!oxu.doesContextExist(ctx)) {
+                throw new InvalidDataException("Context " + ctx.getId() + " does not exist.");
+            }
+        } catch (StorageException e) {
+            throw new InvalidDataException(e);
+        }
+         * 
+         */
     }
 
     private String getUserIdArrayFromUsersAsString(final User[] users) throws InvalidDataException {
@@ -1176,4 +1267,93 @@ public class OXUser extends OXCommonImpl implements OXUserInterface {
         }
         return list.toArray(new User[list.size()]);
     }
+
+    /* (non-Javadoc)
+     * @see com.openexchange.admin.rmi.OXUserInterface#changeModuleAccessGlobal(java.lang.String, com.openexchange.admin.rmi.dataobjects.UserModuleAccess, com.openexchange.admin.rmi.dataobjects.UserModuleAccess, com.openexchange.admin.rmi.dataobjects.Credentials)
+     */
+    public void changeModuleAccessGlobal(final String filter, final UserModuleAccess addAccess, final UserModuleAccess removeAccess, Credentials auth) throws RemoteException, InvalidCredentialsException, StorageException, InvalidDataException {
+        auth = auth == null ? new Credentials("","") : auth;
+        try {
+            doNullCheck(addAccess, removeAccess);
+        } catch (final InvalidDataException e1) {
+            final InvalidDataException invalidDataException = new InvalidDataException("Some parameters are null");
+            log.error(invalidDataException.getMessage(), invalidDataException);
+            throw invalidDataException;
+        }
+
+        if (log.isDebugEnabled()) {
+            log.debug(filter + " - " + addAccess + " - "+ removeAccess + " - " + auth);
+        }
+
+        try {
+            basicauth.doAuthentication(auth);
+        } catch (final InvalidCredentialsException e) {
+            log.error(e.getMessage(), e);
+            throw e;
+        }
+        
+        int permissionBits = -1;
+        if (filter != null) {
+            try {
+                permissionBits = Integer.parseInt(filter);
+            } catch (final NumberFormatException nfe) {
+                final UserModuleAccess namedAccessCombination = ClientAdminThread.cache.getNamedAccessCombination(filter);
+                if (namedAccessCombination == null) {
+                    throw new InvalidDataException("No such access combination name \"" + filter.trim() + "\"");
+                }
+                permissionBits = namedAccessCombination.getPermissionBits();
+            }
+        }
+        
+        final int addBits = addAccess.getPermissionBits();
+        final int removeBits = removeAccess.getPermissionBits();
+        if (log.isDebugEnabled()) {
+            log.debug("Adding " + addBits + " removing " + removeBits + " to filter " + filter);
+        }
+
+        try {
+            tool.changeAccessCombination(permissionBits, addBits, removeBits);
+        } catch (final StorageException e) {
+            log.error(e.getMessage(), e);
+            throw e;
+        }
+
+    }
+
+    public boolean exists(final Context ctx, final User user, Credentials auth) throws RemoteException, InvalidDataException, InvalidCredentialsException, StorageException, DatabaseUpdateException, NoSuchContextException {
+        auth = auth == null ? new Credentials("","") : auth;
+        try {
+            doNullCheck(user);
+        } catch (final InvalidDataException e2) {
+            final InvalidDataException invalidDataException = new InvalidDataException("One of the given arguments for change is null");
+            log.error(invalidDataException.getMessage(), invalidDataException);
+            throw invalidDataException;
+        }
+
+        try {
+            contextcheck(ctx);
+
+            checkContextAndSchema(ctx);
+
+            if( null != user.getId() ) {
+                return tool.existsUser(ctx, user);
+            } else if( null != user.getName() ) {
+                return tool.existsUserName(ctx, user.getName());
+            } else {
+                throw new InvalidDataException("Neither id nor name is set in supplied user object");
+            }
+        } catch (final InvalidCredentialsException e) {
+            log.error(e.getMessage(), e);
+            throw e;
+        } catch (final StorageException e) {
+            log.error(e.getMessage(), e);
+            throw e;
+        } catch (final DatabaseUpdateException e) {
+            log.error(e.getMessage(), e);
+            throw e;
+        } catch (final NoSuchContextException e) {
+            log.error(e.getMessage(), e);
+            throw e;
+        }
+    }
 }
