@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -407,15 +407,16 @@ public abstract class OXToolStorageInterface {
     public abstract boolean isContextEnabled(final Context ctx) throws StorageException;
 
     /**
-     * This method checks if the display name of the given user object is already used in for a system user. This method doesn't check
-     * for display names in the contacts of the users
-     *
+     * This method checks if the display name of the given user object is already used for a system user. This method doesn't check for
+     * display names in the contacts of the users.
+     * 
      * @param ctx
      * @param usr
-     * @return
-     * @throws StorageException
+     * @param userId optional user identifier parameter to exclude the user to change. If not applicable give zero.
+     * @return <code>true</code> if the display name is already in use by some other user.
+     * @throws StorageException if a problem occurs on the storage layer.
      */
-    public abstract boolean existsDisplayName(final Context ctx, final User usr) throws StorageException;
+    public abstract boolean existsDisplayName(final Context ctx, final User usr, final int userId) throws StorageException;
 
     public abstract boolean isMasterDatabase(final int database_id) throws StorageException;
 
@@ -446,5 +447,15 @@ public abstract class OXToolStorageInterface {
     public abstract void validateUserName(String name) throws InvalidDataException;
 
     public abstract void checkValidEmailsInUserObject(User user) throws InvalidDataException;
+    
+    /**
+     * Changes access rights for all users in the Database.
+     * 
+     * @param filter Only users with this access combination are affected. Set to -1 for no filter.
+     * @param addAccess
+     * @param removeAccess
+     * @throws StorageException
+     */
+    public abstract void changeAccessCombination(int filter, int addAccess, int removeAccess) throws StorageException;
 
 }
