@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -300,6 +300,7 @@ public class OXGroupMySQLStorage extends OXGroupSQLStorage implements OXMySQLDef
             // set last modified
             changeLastModifiedOnGroup(ctxId, groupId, con);
             con.commit();
+            log.info("Group " + groupId + " changed!");
         } catch (DataTruncation dt) {
             log.error(AdminCache.DATA_TRUNCATION_ERROR_MSG, dt);
             rollback(con);
@@ -371,9 +372,7 @@ public class OXGroupMySQLStorage extends OXGroupSQLStorage implements OXMySQLDef
                 changeLastModifiedOfGroupMembers(ctx, con, group.getMembers());
             }
             con.commit();
-            if (log.isInfoEnabled()) {
-                log.info("Group " + groupId + " created!");
-            }
+            log.info("Group " + groupId + " created!");
         } catch (DataTruncation dt) {
             log.error(AdminCache.DATA_TRUNCATION_ERROR_MSG, dt);
             rollback(con);
@@ -427,6 +426,9 @@ public class OXGroupMySQLStorage extends OXGroupSQLStorage implements OXMySQLDef
                 stmt2.close();
             }
             con.commit();
+            for (Group group : groups) {
+                log.info("Group " + group.getId() + " deleted!");
+            }
         } catch (SQLException e) {
             log.error("SQL Error", e);
             rollback(con);
