@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -167,6 +167,7 @@ public class OXResourceMySQLStorage extends OXResourceSQLStorage implements OXMy
                 changeLastModified(resource_id, ctx, con);
             }
             con.commit();
+            log.info("Resource " + rid + " changed!");
         } catch (final DataTruncation dt) {
             log.error(AdminCache.DATA_TRUNCATION_ERROR_MSG, dt);
             dorollback(con);
@@ -257,6 +258,7 @@ public class OXResourceMySQLStorage extends OXResourceSQLStorage implements OXMy
             prep_insert.executeUpdate();
 
             con.commit();
+            log.info("Resource " + resID + " created!");
             return resID;
         }catch (final DataTruncation dt){
             log.error(AdminCache.DATA_TRUNCATION_ERROR_MSG, dt);
@@ -307,6 +309,7 @@ public class OXResourceMySQLStorage extends OXResourceSQLStorage implements OXMy
             prep_del.executeUpdate();
 
             con.commit();
+            log.info("Resource " + resource_id + " deleted!");
         } catch (final SQLException e) {
             log.error("SQL Error", e);
             dorollback(con);
@@ -362,6 +365,7 @@ public class OXResourceMySQLStorage extends OXResourceSQLStorage implements OXMy
             prep_del.executeUpdate();
             
             con.commit();
+            log.info("Resource " + resource_id + " deleted!");
         } catch (final SQLException e) {
             log.error("SQL Error", e);
             dorollback(con);
