@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -50,6 +50,8 @@
 package com.openexchange.admin.storage.mysqlStorage;
 
 import static com.openexchange.java.Autoboxing.I;
+import static com.openexchange.java.Autoboxing.i;
+import static com.openexchange.sql.grammar.Constant.PLACEHOLDER;
 import static com.openexchange.tools.sql.DBUtils.closeSQLStuff;
 import java.sql.Connection;
 import java.sql.PreparedStatement;
@@ -58,6 +60,8 @@ import java.sql.SQLException;
 import java.util.ArrayList;
 import java.util.HashSet;
 import java.util.Iterator;
+import java.util.List;
+import java.util.Set;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import com.openexchange.admin.daemons.ClientAdminThread;
@@ -72,13 +76,29 @@ import com.openexchange.admin.rmi.exceptions.EnforceableDataObjectException;
 import com.openexchange.admin.rmi.exceptions.InvalidDataException;
 import com.openexchange.admin.rmi.exceptions.PoolException;
 import com.openexchange.admin.rmi.exceptions.StorageException;
+import com.openexchange.admin.services.AdminServiceRegistry;
 import com.openexchange.admin.storage.sqlStorage.OXToolSQLStorage;
 import com.openexchange.admin.tools.AdminCache;
 import com.openexchange.admin.tools.GenericChecks;
+import com.openexchange.config.cascade.ComposedConfigProperty;
+import com.openexchange.config.cascade.ConfigCascadeException;
+import com.openexchange.config.cascade.ConfigView;
+import com.openexchange.config.cascade.ConfigViewFactory;
+import com.openexchange.database.DBPoolingException;
 import com.openexchange.groupware.container.FolderObject;
+import com.openexchange.groupware.contexts.impl.ContextException;
+import com.openexchange.groupware.contexts.impl.ContextStorage;
 import com.openexchange.groupware.update.UpdateException;
 import com.openexchange.groupware.update.UpdateStatus;
 import com.openexchange.groupware.update.Updater;
+import com.openexchange.sql.builder.StatementBuilder;
+import com.openexchange.sql.grammar.BitAND;
+import com.openexchange.sql.grammar.BitOR;
+import com.openexchange.sql.grammar.Column;
+import com.openexchange.sql.grammar.EQUALS;
+import com.openexchange.sql.grammar.INVERT;
+import com.openexchange.sql.grammar.Table;
+import com.openexchange.sql.grammar.UPDATE;
 
 /**
  * @author d7
@@ -237,35 +257,35 @@ public class OXToolMySQLStorage extends OXToolSQLStorage implements OXMySQLDefau
     }
 
     @Override
-    public boolean existsDisplayName(Context ctx, User user) throws StorageException {
-        final int contextId = ctx.getId().intValue();
+    public boolean existsDisplayName(final Context ctx, final User user, final int userId) throws StorageException {
+        final int ctxId = i(ctx.getId());
         final Connection con;
         try {
-            con = cache.getConnectionForContext(contextId);
-        } catch (PoolException e) {
+            con = cache.getConnectionForContext(ctxId);
+        } catch (final PoolException e) {
             log.error("Pool Error", e);
             throw new StorageException(e);
         }
         PreparedStatement stmt = null;
-        ResultSet result = null;
+        ResultSet rs = null;
         boolean foundOther = false;
         try {
-            stmt = con.prepareStatement("SELECT field01 FROM prg_contacts WHERE cid=? AND field01=? AND fid=?");
-            stmt.setInt(1, contextId);
+            stmt = con.prepareStatement("SELECT field01,userid FROM prg_contacts WHERE cid=? AND field01=? AND fid=?");
+            stmt.setInt(1, ctxId);
             stmt.setString(2, user.getDisplay_name());
             stmt.setInt(3, FolderObject.SYSTEM_LDAP_FOLDER_ID);
-            result = stmt.executeQuery();
-            while (!foundOther && result.next()) {
-                foundOther = user.getDisplay_name().equals(result.getString(1));
+            rs = stmt.executeQuery();
+            while (!foundOther && rs.next()) {
+                foundOther = user.getDisplay_name().equals(rs.getString(1)) && (userId == 0 || userId != rs.getInt(2)); 
             }
-        } catch (SQLException e) {
+        } catch (final SQLException e) {
             log.error("SQL Error", e);
             throw new StorageException(e);
         } finally {
-            closeSQLStuff(result, stmt);
+            closeSQLStuff(rs, stmt);
             try {
-                cache.pushConnectionForContext(contextId, con);
-            } catch (PoolException e) {
+                cache.pushConnectionForContext(ctxId, con);
+            } catch (final PoolException e) {
                 log.error("Error pushing context connection to pool.", e);
             }
         }
@@ -479,9 +499,7 @@ public class OXToolMySQLStorage extends OXToolSQLStorage implements OXMySQLDefau
             closePreparedStatement(prep_check);
 
             try {
-               if(con!=null){
                 cache.pushConnectionForContext(ctx.getId().intValue(), con);
-               }
             } catch (final PoolException e) {
                 log.error("Error pushing ox write connection to pool!", e);
             }
@@ -1133,7 +1151,9 @@ public class OXToolMySQLStorage extends OXToolSQLStorage implements OXMySQLDefau
             closePreparedStatement(prep_check);
 
             try {
-                cache.pushConnectionForContext(ctx.getId().intValue(), con);
+                if(con != null) {
+                    cache.pushConnectionForContext(ctx.getId().intValue(), con);
+                }
             } catch (final PoolException e) {
                 log.error("Error pushing ox db read connection to pool!", e);
             }
@@ -1429,11 +1449,11 @@ public class OXToolMySQLStorage extends OXToolSQLStorage implements OXMySQLDefau
     }
 
     @Override
-    public void primaryMailExists(Context ctx, String mail) throws StorageException, InvalidDataException {
+    public void primaryMailExists(final Context ctx, final String mail) throws StorageException, InvalidDataException {
         final Connection con;
         try {
             con = cache.getConnectionForContext(ctx.getId().intValue());
-        } catch (PoolException e) {
+        } catch (final PoolException e) {
             log.error("Pool Error", e);
             throw new StorageException(e);
         }
@@ -1448,7 +1468,7 @@ public class OXToolMySQLStorage extends OXToolSQLStorage implements OXMySQLDefau
         }
     }
 
-    private void primaryMailExists(Connection con, Context ctx, String mail) throws StorageException, InvalidDataException {
+    private void primaryMailExists(final Connection con, final Context ctx, final String mail) throws StorageException, InvalidDataException {
         PreparedStatement stmt = null;
         ResultSet result = null;
         try {
@@ -1459,7 +1479,7 @@ public class OXToolMySQLStorage extends OXToolSQLStorage implements OXMySQLDefau
             if (result.next()) {
                 throw new InvalidDataException("Primary mail address already exists in this context.");
             }
-        } catch (SQLException e) {
+        } catch (final SQLException e) {
             log.error("SQL Error", e);
             throw new StorageException(e);
         } finally {
@@ -1468,10 +1488,10 @@ public class OXToolMySQLStorage extends OXToolSQLStorage implements OXMySQLDefau
     }
 
     @Override
-    public boolean checkAndUpdateSchemaIfRequired(int contextId) throws StorageException {
+    public boolean checkAndUpdateSchemaIfRequired(final int contextId) throws StorageException {
         try {
             final Updater updater = Updater.getInstance();
-            UpdateStatus status = updater.getStatus(contextId);
+            final UpdateStatus status = updater.getStatus(contextId);
             if (status.blockingUpdatesRunning()) {
                 return true;
             }
@@ -1479,7 +1499,7 @@ public class OXToolMySQLStorage extends OXToolSQLStorage implements OXMySQLDefau
                 updater.startUpdate(contextId);
                 return true;
             }
-        } catch (UpdateException e) {
+        } catch (final UpdateException e) {
             throw new StorageException(e.getMessage(), e);
         }
         return false;
@@ -1489,7 +1509,7 @@ public class OXToolMySQLStorage extends OXToolSQLStorage implements OXMySQLDefau
         Updater updater;
         try {
             updater = Updater.getInstance();
-            UpdateStatus status = updater.getStatus(schema, writePoolId);
+            final UpdateStatus status = updater.getStatus(schema, writePoolId);
             if (status.blockingUpdatesRunning()) {
                 log.info("Another database update process is already running");
                 return true;
@@ -1510,7 +1530,7 @@ public class OXToolMySQLStorage extends OXToolSQLStorage implements OXMySQLDefau
             // either with or without starting an update task, when we reach this point, we
             // must return true
             return true;
-        } catch (UpdateException e) {
+        } catch (final UpdateException e) {
             if (e.getDetailNumber() == 102) {
                 // NOTE: this situation should not happen!
                 // it can only happen, when a schema has not been initialized correctly!
@@ -2099,12 +2119,12 @@ public class OXToolMySQLStorage extends OXToolSQLStorage implements OXMySQLDefau
     }
     
     @Override
-    public boolean existsUserName(Context ctx, User user) throws StorageException {
+    public boolean existsUserName(final Context ctx, final User user) throws StorageException {
         final int contextId = ctx.getId().intValue();
         final Connection con;
         try {
             con = cache.getConnectionForContext(contextId);
-        } catch (PoolException e) {
+        } catch (final PoolException e) {
             log.error("Pool Error", e);
             throw new StorageException(e);
         }
@@ -2120,7 +2140,7 @@ public class OXToolMySQLStorage extends OXToolSQLStorage implements OXMySQLDefau
             while (!foundOther && result.next()) {
                 foundOther = user.getName().equals(result.getString(1));
             }
-        } catch (SQLException e) {
+        } catch (final SQLException e) {
             log.error("SQL Error", e);
             throw new StorageException(e);
         } finally {
@@ -2142,8 +2162,8 @@ public class OXToolMySQLStorage extends OXToolSQLStorage implements OXMySQLDefau
     }
 
     @Override
-    public void checkCreateUserData(Context ctx, User usr) throws InvalidDataException, EnforceableDataObjectException, StorageException {
-        checkAndSetLanguage(usr);
+    public void checkCreateUserData(final Context ctx, final User usr) throws InvalidDataException, EnforceableDataObjectException, StorageException {
+        checkAndSetLanguage(ctx, usr);
         GenericChecks.checkCreateValidPasswordMech(usr);
         if (usr.getPassword() == null || usr.getPassword().trim().length() == 0) {
             throw new InvalidDataException("Empty password is not allowed.");
@@ -2151,8 +2171,8 @@ public class OXToolMySQLStorage extends OXToolSQLStorage implements OXMySQLDefau
         if (!usr.mandatoryCreateMembersSet()) {
             throw new InvalidDataException("Mandatory fields not set: " + usr.getUnsetMembers());
         }
-        if (!usr.isContextadmin() && prop.getUserProp(AdminProperties.User.DISPLAYNAME_UNIQUE, true)) {
-            if (existsDisplayName(ctx, usr)) {
+        if (!usr.isContextadmin()) {
+            if (existsDisplayName(ctx, usr, 0)) {
                 throw new InvalidDataException("The displayname is already used");
             }
         }
@@ -2180,12 +2200,12 @@ public class OXToolMySQLStorage extends OXToolSQLStorage implements OXMySQLDefau
         usr.addAlias(usr.getPrimaryEmail());
         // Check mail attributes
         {
-            HashSet<String> useraliases = usr.getAliases();
-            String primaryEmail = usr.getPrimaryEmail();
-            String email1 = usr.getEmail1();
-            boolean foundPrimaryMail = useraliases.contains(primaryEmail);
-            boolean foundEmail1 = useraliases.contains(email1);
-            boolean foundDefaultSenderAddress = useraliases.contains(usr.getDefaultSenderAddress());
+            final HashSet<String> useraliases = usr.getAliases();
+            final String primaryEmail = usr.getPrimaryEmail();
+            final String email1 = usr.getEmail1();
+            final boolean foundPrimaryMail = useraliases.contains(primaryEmail);
+            final boolean foundEmail1 = useraliases.contains(email1);
+            final boolean foundDefaultSenderAddress = useraliases.contains(usr.getDefaultSenderAddress());
             if (!foundPrimaryMail || !foundEmail1 || !foundDefaultSenderAddress) {
                 throw new InvalidDataException("primaryMail, Email1 and defaultSenderAddress must be present in set of aliases.");
             }
@@ -2197,9 +2217,45 @@ public class OXToolMySQLStorage extends OXToolSQLStorage implements OXMySQLDefau
         }
     }
 
-    private void checkAndSetLanguage(User user) throws InvalidDataException {
+    private void checkAndSetLanguage(final Context ctx, final User user) throws InvalidDataException {
         String lang = user.getLanguage();
         if (lang == null) {
+            final ConfigViewFactory viewFactory = AdminServiceRegistry.getInstance().getService(ConfigViewFactory.class);
+            if (viewFactory != null) {
+                try {
+                    int ctxId = ctx.getId();
+                    if (user.isContextadmin() && user.getId() == null) {
+                        ctxId = -1;
+                    }
+                    final ConfigView view = viewFactory.getView(-1, ctxId);
+                    final String userAttribute = ctx.getUserAttribute("config", "com.openexchange.admin.user.defaultLanguage");
+                    if (userAttribute != null) {
+                        final ComposedConfigProperty<String> property = view.property("com.openexchange.admin.user.defaultLanguage", String.class).precedence(
+                            "server, contextSets");
+                        if (property.isDefined()) {
+                            lang = property.get();
+                        } else {
+                            lang = userAttribute;
+                        }
+                    } else {
+                        final ComposedConfigProperty<String> property = view.property("com.openexchange.admin.user.defaultLanguage", String.class);
+                        if (property.isDefined()) {
+                            lang = property.get();
+                        }
+                    }
+                    
+                    if (lang != null) {
+                        if (lang.indexOf('_') == -1) {
+                            throw new InvalidDataException("language must contain an underscore, e.g. en_US");
+                        }
+                        user.setLanguage(lang);
+                        return;
+                    }
+                } catch (final ConfigCascadeException e) {
+                    throw new InvalidDataException(e.getMessage());
+                }
+                
+            }
             user.setLanguage(FALLBACK_LANGUAGE_CREATE + '_' + FALLBACK_COUNTRY_CREATE);
         } else {
             if (lang.indexOf('_') == -1) {
@@ -2209,28 +2265,92 @@ public class OXToolMySQLStorage extends OXToolSQLStorage implements OXMySQLDefau
     }
 
     @Override
-    public void validateUserName(String userName) throws InvalidDataException {
+    public void validateUserName(final String userName) throws InvalidDataException {
         // Check for allowed chars:
         // abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_-+.%$@
-        String usrUidRegexp = prop.getUserProp("CHECK_USER_UID_REGEXP", "[$@%\\.+a-zA-Z0-9_-]");        
-        String illegal = userName.replaceAll(usrUidRegexp,"");
+        final String usrUidRegexp = prop.getUserProp("CHECK_USER_UID_REGEXP", "[$@%\\.+a-zA-Z0-9_-]");        
+        final String illegal = userName.replaceAll(usrUidRegexp,"");
         if (illegal.length() > 0) {
             throw new InvalidDataException("Illegal chars: \"" + illegal + "\"");
         }
     }
 
     @Override
-    public void checkValidEmailsInUserObject(User usr) throws InvalidDataException {
+    public void checkValidEmailsInUserObject(final User usr) throws InvalidDataException {
         GenericChecks.checkValidMailAddress(usr.getPrimaryEmail());
         GenericChecks.checkValidMailAddress(usr.getEmail1());
         GenericChecks.checkValidMailAddress(usr.getEmail2());
         GenericChecks.checkValidMailAddress(usr.getEmail3());
         GenericChecks.checkValidMailAddress(usr.getDefaultSenderAddress());
-        HashSet<String> aliases = usr.getAliases();
+        final HashSet<String> aliases = usr.getAliases();
         if (aliases != null) {
             for (final String addr : aliases) {
                 GenericChecks.checkValidMailAddress(addr);
             }
         }
     }
+
+    @Override
+    public void changeAccessCombination(final int filter, final int addAccess, final int removeAccess) throws StorageException {
+        
+        //Collecting ONE Context id for each schema
+        final Set<Integer> contextIdsForSchema = new HashSet<Integer>();
+        try {
+            final List<Integer> allCids = ContextStorage.getInstance().getAllContextIds();
+            final Set<Integer> handledContextIds = new HashSet<Integer>();
+            for (final int cid : allCids) {
+                if (!handledContextIds.contains(cid)) {
+                    final int[] contextsInSameSchema = com.openexchange.databaseold.Database.getContextsInSameSchema(cid);
+                    for (final int contextInSameSchama : contextsInSameSchema) {
+                        handledContextIds.add(contextInSameSchama);
+                    }
+                    contextIdsForSchema.add(cid);
+                }
+            }
+        } catch (final DBPoolingException e) {
+            log.error("DBPool Error", e);
+            throw new StorageException(e);
+        } catch (final ContextException e) {
+            log.error("Context Storage Error", e);
+            throw new StorageException(e);
+        }
+        
+        //Execute once for each schema
+        for (final int cid : contextIdsForSchema) {
+            changeAccessCombination(cid, filter, addAccess, removeAccess);
+        }
+    }
+
+    private void changeAccessCombination(final int cid, final int filter, final int addAccess, final int removeAccess) throws StorageException {
+        Connection con = null;
+        final Table table = new Table("user_configuration");
+        final Column column = new Column("permissions");
+        
+        try {
+            con = cache.getConnectionForContextNoTimeout(cid);
+            final UPDATE update = new UPDATE(table).SET(column, new BitAND(new BitOR(column, PLACEHOLDER), new INVERT(PLACEHOLDER)));
+            
+            final List<Object> values = new ArrayList<Object>();
+            values.add(addAccess);
+            values.add(removeAccess);
+            
+            if (filter != -1) {
+                update.WHERE(new EQUALS(column, PLACEHOLDER));
+                values.add(filter);
+            }
+            new StatementBuilder().executeStatement(con, update, values);
+        } catch (final PoolException e) {
+            log.error("Pool Error", e);
+            throw new StorageException(e);
+        } catch (final SQLException e) {
+            log.error("SQL Error", e);
+            throw new StorageException(e);
+        } finally {
+            try {
+                cache.pushConnectionForContextNoTimeout(cid, con);
+            } catch (final PoolException e) {
+                log.error("Error pushing context connection to pool.", e);
+            }
+        }
+    }
 }
