@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -82,11 +82,11 @@ public class OXUtilMySQLStorageCommon {
 
     private static AdminCache cache = ClientAdminThread.cache;
 
-    public void createDatabase(Database db) throws StorageException {
+    public void createDatabase(final Database db) throws StorageException {
         final List<String> createTableStatements;
         try {
             createTableStatements = cache.getOXDBInitialQueries();
-        } catch (OXGenericException e) {
+        } catch (final OXGenericException e) {
             LOG.error("Error reading DB init Queries!", e);
             throw new StorageException(e);
         }
@@ -96,11 +96,11 @@ public class OXUtilMySQLStorageCommon {
             sql_pass = db.getPassword();
         }
         try {
-            con = cache.getSimpleSqlConnection(db.getUrl(), db.getLogin(), sql_pass, db.getDriver());
-        } catch (SQLException e) {
+            con = cache.getSimpleSQLConnectionWithoutTimeout(db.getUrl(), db.getLogin(), sql_pass, db.getDriver());
+        } catch (final SQLException e) {
             LOG.error("SQL Error", e);
             throw new StorageException(e.toString(), e);
-        } catch (ClassNotFoundException e) {
+        } catch (final ClassNotFoundException e) {
             LOG.error("Driver not found to create database ", e);
             throw new StorageException(e);
         }
@@ -117,15 +117,15 @@ public class OXUtilMySQLStorageCommon {
             con.setCatalog(db.getScheme());
             pumpData2DatabaseOld(con, createTableStatements);
             pumpData2DatabaseNew(con, CreateTableRegistry.getInstance().getList());
-            initUpdateTaskTable(con);
+            initUpdateTaskTable(con, db.getId().intValue(), db.getScheme());
             con.commit();
-        } catch (SQLException e) {
+        } catch (final SQLException e) {
             rollback(con);
             if (created) {
                 deleteDatabase(con, db);
             }
             throw new StorageException(e.toString());
-        } catch (StorageException e) {
+        } catch (final StorageException e) {
             rollback(con);
             if (created) {
                 deleteDatabase(con, db);
@@ -137,7 +137,7 @@ public class OXUtilMySQLStorageCommon {
         }
     }
 
-    private boolean existsDatabase(Connection con, String name) throws StorageException {
+    private boolean existsDatabase(final Connection con, final String name) throws StorageException {
         PreparedStatement stmt = null;
         ResultSet result = null;
         try {
@@ -145,7 +145,7 @@ public class OXUtilMySQLStorageCommon {
             stmt.setString(1, name);
             result = stmt.executeQuery();
             return result.next();
-        } catch (SQLException e) {
+        } catch (final SQLException e) {
             LOG.error("SQL Error", e);
             throw new StorageException(e.getMessage(), e);
         } finally {
@@ -153,12 +153,12 @@ public class OXUtilMySQLStorageCommon {
         }
     }
 
-    private void createDatabase(Connection con, String name) throws StorageException {
+    private void createDatabase(final Connection con, final String name) throws StorageException {
         Statement stmt = null;
         try {
             stmt = con.createStatement();
             stmt.executeUpdate("CREATE DATABASE `" + name + "` DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci");
-        } catch (SQLException e) {
+        } catch (final SQLException e) {
             LOG.error("SQL Error", e);
             throw new StorageException(e.getMessage(), e);
         } finally {
@@ -166,54 +166,88 @@ public class OXUtilMySQLStorageCommon {
         }
     }
 
-    private void pumpData2DatabaseOld(Connection con, List<String> db_queries) throws StorageException {
+    private void pumpData2DatabaseOld(final Connection con, final List<String> db_queries) throws StorageException {
         Statement stmt = null;
         try {
-            stmt = con.createStatement();
-            for (String sqlCreate : db_queries) {
+            try {
+                stmt = con.createStatement();
+            } catch (final SQLException e) {
+                throw new StorageException(e.getMessage(), e);
+            }
+            for (final String sqlCreate : db_queries) {
                 stmt.addBatch(sqlCreate);
             }
             stmt.executeBatch();
-        } catch (SQLException e) {
-            LOG.error("SQL Error", e);
-            throw new StorageException(e.getMessage(), e);
+        } catch (final SQLException e) {
+            if (e.getMessage().indexOf("already exists") < 0) { // MySQL error: "PROCEDURE get_mail_service_id already exists"
+                throw new StorageException(e.getMessage(), e);
+            }
+            closeSQLStuff(stmt);
+            stmt = null;
+            if (LOG.isDebugEnabled()) {
+                LOG.info("Batch table creation failed.", e);
+            } else {
+                LOG.info("Batch table creation failed.");
+            }
+            /*
+             * Execute them one-by-one...
+             */
+            try {
+                for (final String sqlCreate : db_queries) {
+                    stmt = con.createStatement();
+                    try {
+                        stmt.executeUpdate(sqlCreate);
+                    } catch (final SQLException sqlException) {
+                        if (LOG.isDebugEnabled()) {
+                            LOG.warn("Following SQL CREATE statement failed with \"" + sqlException.getMessage() + "\": "  + sqlCreate, sqlException);
+                        } else {
+                            LOG.warn("Following SQL CREATE statement failed with \"" + sqlException.getMessage() + "\": "  + sqlCreate);
+                        }
+                    } finally {
+                        closeSQLStuff(stmt);
+                        stmt = null;
+                    }
+                }
+            } catch (final SQLException abort) {
+                throw new StorageException(abort.getMessage(), abort);
+            }
         } finally {
             closeSQLStuff(stmt);
         }
     }
 
-    private void pumpData2DatabaseNew(Connection con, List<CreateTableService> createTables) throws StorageException {
-        Set<String> existingTables = new HashSet<String>();
-        for (String table : FROM_SCRIPTS) {
+    private void pumpData2DatabaseNew(final Connection con, final List<CreateTableService> createTables) throws StorageException {
+        final Set<String> existingTables = new HashSet<String>();
+        for (final String table : FROM_SCRIPTS) {
             existingTables.add(table);
         }
-        List<CreateTableService> toCreate = new ArrayList<CreateTableService>(createTables.size());
+        final List<CreateTableService> toCreate = new ArrayList<CreateTableService>(createTables.size());
         toCreate.addAll(createTables);
         CreateTableService next;
         try {
             while ((next = findNext(toCreate, existingTables)) != null) {
                 next.perform(con);
-                for (String createdTable : next.tablesToCreate()) {
+                for (final String createdTable : next.tablesToCreate()) {
                     existingTables.add(createdTable);
                 }
                 toCreate.remove(next);
             }
-        } catch (AbstractOXException e) {
+        } catch (final AbstractOXException e) {
             throw new StorageException(e.getMessage(), e);
         }
         if (!toCreate.isEmpty()) {
-            StringBuilder sb = new StringBuilder();
+            final StringBuilder sb = new StringBuilder();
             sb.append("Unable to determine next CreateTableService to execute.\n");
             sb.append("Existing tables: ");
-            for (String existingTable : existingTables) {
+            for (final String existingTable : existingTables) {
                 sb.append(existingTable);
                 sb.append(',');
             }
             sb.setCharAt(sb.length() - 1, '\n');
-            for (CreateTableService service : toCreate) {
+            for (final CreateTableService service : toCreate) {
                 sb.append(service.getClass().getName());
                 sb.append(": ");
-                for (String tableToCreate : service.requiredTables()) {
+                for (final String tableToCreate : service.requiredTables()) {
                     sb.append(tableToCreate);
                     sb.append(',');
                 }
@@ -224,10 +258,10 @@ public class OXUtilMySQLStorageCommon {
         }
     }
 
-    private CreateTableService findNext(List<CreateTableService> toCreate, Set<String> existingTables) {
-        for (CreateTableService service : toCreate) {
-            List<String> requiredTables = new ArrayList<String>();
-            for (String requiredTable : service.requiredTables()) {
+    private CreateTableService findNext(final List<CreateTableService> toCreate, final Set<String> existingTables) {
+        for (final CreateTableService service : toCreate) {
+            final List<String> requiredTables = new ArrayList<String>();
+            for (final String requiredTable : service.requiredTables()) {
                 requiredTables.add(requiredTable);
             }
             if (existingTables.containsAll(requiredTables)) {
@@ -237,26 +271,26 @@ public class OXUtilMySQLStorageCommon {
         return null;
     }
 
-    private void initUpdateTaskTable(Connection con) throws StorageException {
-        UpdateTask[] tasks = Updater.getInstance().getAvailableUpdateTasks();
-        SchemaStore store = SchemaStore.getInstance();
+    private void initUpdateTaskTable(final Connection con, final int poolId, final String schema) throws StorageException {
+        final UpdateTask[] tasks = Updater.getInstance().getAvailableUpdateTasks();
+        final SchemaStore store = SchemaStore.getInstance();
         try {
-            for (UpdateTask task : tasks) {
-                store.addExecutedTask(con, task.getClass().getName(), true);
+            for (final UpdateTask task : tasks) {
+                store.addExecutedTask(con, task.getClass().getName(), true, poolId, schema);
             }
-        } catch (SchemaException e) {
+        } catch (final SchemaException e) {
             throw new StorageException(e.getMessage(), e);
         }
     }
 
-    public void deleteDatabase(Database db) throws StorageException {
+    public void deleteDatabase(final Database db) throws StorageException {
         final Connection con;
         try {
-            con = cache.getSimpleSqlConnection(db.getUrl(), db.getLogin(), db.getPassword(), db.getDriver());
-        } catch (SQLException e) {
+            con = cache.getSimpleSQLConnectionWithoutTimeout(db.getUrl(), db.getLogin(), db.getPassword(), db.getDriver());
+        } catch (final SQLException e) {
             LOG.error("SQL Error", e);
             throw new StorageException(e.toString(), e);
-        } catch (ClassNotFoundException e) {
+        } catch (final ClassNotFoundException e) {
             LOG.error("Driver not found to create database ", e);
             throw new StorageException(e);
         }
@@ -267,14 +301,14 @@ public class OXUtilMySQLStorageCommon {
         }
     }
 
-    private void deleteDatabase(final Connection con, Database db) throws StorageException {
+    private void deleteDatabase(final Connection con, final Database db) throws StorageException {
         Statement stmt = null;
         try {
             con.setAutoCommit(false);
             stmt = con.createStatement();
             stmt.executeUpdate("DROP DATABASE IF EXISTS `" + db.getScheme() + "`");
             con.commit();
-        } catch (SQLException e) {
+        } catch (final SQLException e) {
             rollback(con);
             throw new StorageException(e.getMessage(), e);
         } finally {
