@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,14 +49,14 @@
 
 package com.openexchange.admin.storage.sqlStorage;
 
-import com.openexchange.admin.rmi.exceptions.PoolException;
-import com.openexchange.admin.tools.PropertyHandler;
-import com.openexchange.database.DBPoolingException;
-import com.openexchange.databaseold.Database;
 import java.sql.Connection;
 import java.sql.SQLException;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+import com.openexchange.admin.rmi.exceptions.PoolException;
+import com.openexchange.admin.tools.PropertyHandler;
+import com.openexchange.database.DBPoolingException;
+import com.openexchange.databaseold.Database;
 
 public class OXAdminPoolDBPool implements OXAdminPoolInterface {
 
@@ -122,7 +122,9 @@ public class OXAdminPoolDBPool implements OXAdminPoolInterface {
             log.error("Error pushing context database write connection to pool!", e);
             throw new PoolException(e.getMessage());
         } finally {
-            Database.back(contextId, true, con);
+            if (null != con) {
+                Database.back(contextId, true, con);
+            }
         }
         return true;
     }
@@ -140,4 +142,15 @@ public class OXAdminPoolDBPool implements OXAdminPoolInterface {
         }
         return true;
     }
+
+    public int getServerId() throws PoolException {
+        final int serverId;
+        try {
+            serverId = Database.getServerId();
+        } catch (DBPoolingException e) {
+            log.error("Error getting the identifier of the server! This is normal until at least one server is configured.", e);
+            throw new PoolException(e.getMessage());
+        }
+        return serverId;
+    }
 }
