@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -58,6 +58,7 @@ import java.io.IOException;
 import java.io.UnsupportedEncodingException;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
+import java.net.URLDecoder;
 import java.security.NoSuchAlgorithmException;
 import java.sql.Connection;
 import java.sql.DataTruncation;
@@ -75,10 +76,8 @@ import java.util.Properties;
 import java.util.StringTokenizer;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
-
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
-
 import com.openexchange.admin.exceptions.OXGenericException;
 import com.openexchange.admin.properties.AdminProperties;
 import com.openexchange.admin.rmi.dataobjects.Context;
@@ -149,9 +148,7 @@ public class AdminCache {
         this.prop = new PropertyHandler(System.getProperties());
         cacheSqlScripts();
         configureAuthentication(); // disabling authentication mechs
-        if (!masterAuthenticationDisabled) {
-            readMasterCredentials();
-        }
+        readMasterCredentials();
         this.log.info("Init Cache");
         initPool();
         this.adminCredentialsCache = new Hashtable<Integer, Credentials>();
@@ -188,16 +185,15 @@ public class AdminCache {
             Iterator<String> names = named_access_combinations.keySet().iterator();
             String retval = null;
             while(names.hasNext()){
-                String combi_name = (String) names.next();
+                String combi_name = names.next();
                 if(named_access_combinations.get(combi_name).equals(access_combination)){
                     retval =  combi_name;
                     break;
                 }
             }
             return retval;
-        } else {
-            return null;
         }
+        return null;
     }
 
     public void initAccessCombinations() throws ClassNotFoundException, OXGenericException  {
@@ -238,18 +234,16 @@ public class AdminCache {
 
                         // TODO: Lets stop the admin daemon
                         throw new OXGenericException("Invalid access combinations found in config file!");
-                    } else {
-                        Method meth = module_method_mapping.get(module);
-                        try {
-                            meth.invoke(us, true);
-                        } catch (IllegalArgumentException e) {
-                            log.error("Illegal argument passed to method!", e);
-                        } catch (IllegalAccessException e) {
-                            log.error("Illegal access!", e);
-                        } catch (InvocationTargetException e) {
-                            log.error("Invocation target error!", e);
-                        }
-
+                    }
+                    Method meth = module_method_mapping.get(module);
+                    try {
+                        meth.invoke(us, true);
+                    } catch (IllegalArgumentException e) {
+                        log.error("Illegal argument passed to method!", e);
+                    } catch (IllegalAccessException e) {
+                        log.error("Illegal access!", e);
+                    } catch (InvocationTargetException e) {
+                        log.error("Invocation target error!", e);
                     }
                 }
                 // add moduleaccess object to hashmap/list identified by
@@ -281,7 +275,7 @@ public class AdminCache {
                 if (meth_name.startsWith("set")) {
                     // remember all valid modules and its set methods
                     String module_name = meth_name.substring(3, meth_name.length()).toLowerCase();
-                    if (BLACKLIST != null && !BLACKLIST.contains(module_name)) {
+                    if (!BLACKLIST.contains(module_name)) {
                         module_method_mapping.put(module_name, method);
                         log.debug(module_name);
                     }
@@ -349,6 +343,9 @@ public class AdminCache {
     }
 
     public boolean pushConnectionForContext(final int context_id, final Connection con) throws PoolException {
+        if(con == null) {
+            return true;
+        }
         return this.pool.pushConnectionForContext(context_id, con);
     }
 
@@ -364,6 +361,10 @@ public class AdminCache {
         return this.pool.pushConnectionForConfigDB(con);
     }
 
+    public int getServerId() throws PoolException {
+        return this.pool.getServerId();
+    }
+
     /**
      * @return the adminCredentials
      */
@@ -433,6 +434,37 @@ public class AdminCache {
         return DriverManager.getConnection(url, user, password);
     }
 
+    private static final Pattern pattern = Pattern.compile("[\\?\\&]([\\p{ASCII}&&[^=\\&]]*)=([\\p{ASCII}&&[^=\\&]]*)");
+
+    public Connection getSimpleSQLConnectionWithoutTimeout(String url, String user, String password, String driver) throws SQLException, ClassNotFoundException {
+        Class.forName(driver);
+        DriverManager.setLoginTimeout(120);
+        final int paramStart = url.indexOf('?');
+        final String newUrl;
+        final Properties props = new Properties();
+        props.put("user", user);
+        props.put("password", password);
+        if (-1 != paramStart) {
+            final Matcher matcher = pattern.matcher(url);
+            newUrl = url.substring(0, paramStart);
+            while (matcher.find()) {
+                final String name = matcher.group(1);
+                final String value = matcher.group(2);
+                if (name != null && name.length() > 0 && value != null && value.length() > 0 && !name.toLowerCase().endsWith("timeout")) {
+                    try {
+                        props.put(name, URLDecoder.decode(value, "UTF-8"));
+                    } catch (final UnsupportedEncodingException e) {
+                        // Should not happen for UTF-8.
+                        log.error(e.getMessage(), e);
+                    }
+                }
+            }
+        } else {
+            newUrl = url;
+        }
+        return DriverManager.getConnection(newUrl, props);
+    }
+
     public void closeSimpleConnection(Connection con) {
         if (con != null) {
             try {
